"""AutoCoder Completer Module

This module provides command completion functionality for the AutoCoder CLI.
"""

from .command_completer_v2 import CommandCompleterV2

__all__ = ['CommandCompleterV2'] 